using System;
using System.Collections.Generic;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Threading;

namespace COM2LAN
{
    public struct BackWorkerInfo
    {
        public string comName;
        public int comBaudrate;
    }

    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();

            UpdateComPorts();
        }

        private void btnExit_Click(object sender, EventArgs e)
        {
            if (backgroundWorker1.IsBusy)
                backgroundWorker1.CancelAsync();

            Application.Exit();
        }

        private void btnStart_Click(object sender, EventArgs e)
        {
            if (btnStart.Text != "START")
            { // stop it
                backgroundWorker1.CancelAsync();
            }
            else
            { // start it
                tbLog.Clear();

                btnStart.BackColor = Color.Red;
                btnStart.Text = "STOP";

                grBoxConfig.Enabled = false;

                BackWorkerInfo info;
                info.comName = (string)comboBoxComPort.SelectedItem;
                info.comBaudrate = (int)nUpDnComBaud.Value;

                backgroundWorker1.RunWorkerAsync(info);
            }
        }

        private void UpdateComPorts()
        {
            comboBoxComPort.Items.Clear();

            // add available COM ports
            comboBoxComPort.Items.AddRange(System.IO.Ports.SerialPort.GetPortNames());
        }

        private void comboBoxComPort_DropDown(object sender, EventArgs e)
        {
            UpdateComPorts();
        }

        private void UpdateLog(string str)
        {
            tbLog.AppendText(str);
            tbLog.SelectionStart = tbLog.Text.Length;
            tbLog.ScrollToCaret();
            tbLog.Refresh();
        }

        private void backgroundWorker1_DoWork(object sender, DoWorkEventArgs e)
        {
            BackWorkerInfo info = (BackWorkerInfo)e.Argument;
            const string leftBtnStr = "LEFT BUTTON PRESSED";
            const string rightBtnStr = "RIGHT BUTTON PRESSED";

            int cntLeftBtnStr = 0;
            int cntRightBtnStr = 0;

            try
            {
                // Set up COM port
                System.IO.Ports.SerialPort sPort = new System.IO.Ports.SerialPort(info.comName, info.comBaudrate);

                sPort.Open();
                backgroundWorker1.ReportProgress(0, "Port " + info.comName + "@" + info.comBaudrate.ToString() + " opened." + Environment.NewLine);

                while (!backgroundWorker1.CancellationPending)
                {
                    if (sPort.BytesToRead > 0)
                    {
                        byte[] buf = new byte[sPort.BytesToRead];

                        sPort.Read(buf, 0, buf.Length);

                        for (int i = 0; i < buf.Length; i++)
                        {
                            if (buf[i] == (byte)leftBtnStr[cntLeftBtnStr])
                            {
                                if (++cntLeftBtnStr == leftBtnStr.Length)
                                {
                                    SendKeys.SendWait("{LEFT}");
                                    cntLeftBtnStr = 0;
                                }
                            }
                            else
                            {
                                cntLeftBtnStr = 0;
                            }

                            if (buf[i] == (byte)rightBtnStr[cntRightBtnStr])
                            {
                                if (++cntRightBtnStr == rightBtnStr.Length)
                                {
                                    SendKeys.SendWait("{RIGHT}");
                                    cntRightBtnStr = 0;
                                }
                            }
                            else
                            {
                                cntRightBtnStr = 0;
                            }

                        }

                        backgroundWorker1.ReportProgress(0, System.Text.ASCIIEncoding.ASCII.GetString(buf));
                    }

                    Thread.Sleep(10);
                }

                // close the serial port
                sPort.Close();
            }
            catch (InvalidOperationException exp)
            {

            }
            catch (ArgumentException exp) 
            { 
            }
            catch
            {
                backgroundWorker1.ReportProgress(0, "Unknown exception!");
            }
            finally
            {                
                backgroundWorker1.ReportProgress(0, "Server stopped!" + Environment.NewLine);
            }
        }

        private void backgroundWorker1_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            UpdateLog((string)e.UserState);
        }

        private void backgroundWorker1_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            btnStart.BackColor = Color.Lime;
            btnStart.Text = "START";

            grBoxConfig.Enabled = true;
        }

        private void Form1_SizeChanged(object sender, EventArgs e)
        {
            tbLog.Width = this.Width - 20 - 30;
            tbLog.Height = this.Height - 90 - 90;
        }

        private void btnClearLog_Click(object sender, EventArgs e)
        {
            tbLog.Clear();
        }
    }
}